
void TM::RenderFilled(PPC *ppc, FrameBuffer *fb) {

	for (int vi = 0; vi < vertsN; vi++) {
//#if GPU
		VertexShader(verts[vi]); // GPU
//#else
		ppc->Project(verts[vi], pverts[vi]); // SW
//#endif
	}

	for (int tri = 0; tri < trisN; tri++) {

// #if GPU
		GeometryShader(triangle tri);
// #endif
		unsigned int vis[3] = { tris[3 * tri + 0], tris[3 * tri + 1], tris[3 * tri + 2] };
		if (
			pverts[vis[0]][0] == FLT_MAX ||
			pverts[vis[1]][0] == FLT_MAX ||
			pverts[vis[2]][0] == FLT_MAX // implement clipping
			)
			continue;
		AABB aabb(pverts[vis[0]]);
		aabb.AddPoint(pverts[vis[1]]);
		aabb.AddPoint(pverts[vis[2]]);
		if (!aabb.Clip2D(0.0f, (float)fb->w, 0.0f, (float)fb->h))
			continue;
		int lefti = (int)(aabb.c0[0] + 0.5f);
		int righti = (int)(aabb.c1[0] - 0.5f);
		int topi = (int)(aabb.c0[1] + 0.5f);
		int bottomi = (int)(aabb.c1[1] - 0.5f);

		//setup edge equations
		M33 eeqs;
		for (int ei = 0; ei < 3; ei++) {
			V3 v0 = pverts[vis[ei]];
			V3 v1 = pverts[vis[(ei + 1) % 3]];
			V3 v2 = pverts[vis[(ei + 2) % 3]]; v2[2] = 1.0f;
			eeqs[ei][0] = v0[1] - v1[1];
			eeqs[ei][1] = v1[0] - v0[0];
			eeqs[ei][2] = -v0[0] * eeqs[ei][0] - v0[1] * eeqs[ei][1];
			if (eeqs[ei] * v2 < 0.0f)
				eeqs[ei] = eeqs[ei] * -1.0f;
		}

		M33 ssi;
		ssi[0] = pverts[vis[0]];
		ssi[1] = pverts[vis[1]];
		ssi[2] = pverts[vis[2]];
		V3 pvzs = ssi.GetColumn(2);
		ssi.SetColumn(2, V3(1.0f, 1.0f, 1.0f));
		ssi = ssi.Inverted();

		V3 zABC = ssi * pvzs;
		M33 colsm;
		colsm[0] = colors[vis[0]];
		colsm[1] = colors[vis[1]];
		colsm[2] = colors[vis[2]];
		M33 colsABC = ssi * colsm;
		colsABC = colsABC.Transposed();

		M33 normsm;
		normsm[0] = normals[vis[0]];
		normsm[1] = normals[vis[1]];
		normsm[2] = normals[vis[2]];
		M33 normsABC = ssi * normsm;
		normsABC = normsABC.Transposed();


		for (int v = topi; v <= bottomi; v++) {
			for (int u = lefti; u <= righti; u++) {
				V3 pc(.5f + (float)u, .5f + (float)v, 1.0f);
				V3 ss = eeqs * pc;
				if (ss[0] < 0.0f || ss[1] < 0.0f || ss[2] < 0.0f)
					continue;
				float currz = pc * zABC;
				if (!fb->Visible(u, v, currz))
					continue;
				V3 currCol = colsABC * pc;
				//				currCol = (refRay + V3(1.0f, 1.0f, 1.0f)) * 0.5f;
// if GPU
			PIXELSHADER(currz, currCol, ...)
// #else
				fb->Set(u, v, currCol.GetColor());
// ENDIF
			}
		}
	}

}
